# (C) 2001-2012 Altera Corporation. All rights reserved.
# Your use of Altera Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Altera Program License Subscription 
# Agreement, Altera MegaCore Function License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Altera and sold by 
# Altera or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


# +-----------------------------------
# | request TCL package from ACDS 9.1
# |
package require -exact sopc 9.1
# |
# +-----------------------------------

# +-----------------------------------
# | module altera_eth_10g_mac
# |
set_module_property NAME altera_eth_10g_mac
set_module_property AUTHOR "Altera Corporation"
set_module_property VERSION 11.1
set_module_property INTERNAL false
set_module_property GROUP "Interface Protocols/Ethernet"
set_module_property DISPLAY_NAME "Ethernet 10G MAC"
set_module_property DESCRIPTION "Altera Ethernet 10G MAC"
set_module_property DATASHEET_URL "http://www.altera.com/literature/ug/10Gbps_MAC.pdf"
set_module_property INSTANTIATE_IN_SYSTEM_MODULE true
set_module_property EDITABLE false
set_module_property COMPOSE_CALLBACK compose
set_module_property ANALYZE_HDL false
set_module_property VALIDATION_CALLBACK validate
# |
# +-----------------------------------

# +-----------------------------------
# | parameters
# |
source altera_eth_10g_mac_conf.tcl

add_parameter DEVICE_FAMILY STRING "Stratix IV"
set_parameter_property DEVICE_FAMILY DISPLAY_NAME "Device Family"
set_parameter_property DEVICE_FAMILY DESCRIPTION "Target Device Family for Ethernet 10G MAC IP"
set_parameter_property DEVICE_FAMILY ALLOWED_RANGES {"Arria GX" "Arria II GX" "Arria II GZ" "Cyclone IV GX" "HardCopy IV" "Stratix II GX" "Stratix III" "Stratix IV"  "Stratix V" }
set_parameter_property DEVICE_FAMILY VISIBLE false
set_parameter_property DEVICE_FAMILY IS_HDL_PARAMETER false
set_parameter_property DEVICE_FAMILY SYSTEM_INFO DEVICE_FAMILY

add_display_item "System Info" DEVICE_FAMILY parameter


# |
# +-----------------------------------


sopc::preview_add_transform "foo" "PREVIEW_AVALON_MM_TRANSFORM"

proc validate {} {
    ethernet_10g_mac_update_gui
}

proc compose {} {
    # Get parameter values
    set datapath_option [get_parameter_value DATAPATH_OPTION]
    set supp_addr_ena [get_parameter_value ENABLE_SUPP_ADDR]
    set pfc_ena [get_parameter_value ENABLE_PFC]
    set pfc_priority_num [get_parameter_value PFC_PRIORITY_NUM]
    set stat_ena [get_parameter_value INSTANTIATE_STATISTICS]
    set reg_stat [get_parameter_value REGISTER_BASED_STATISTICS]
    set crc_tx [get_parameter_value INSTANTIATE_TX_CRC]
    set preamble_mode [get_parameter_value PREAMBLE_PASSTHROUGH]
    
    
    
    # Common for TX & RX
    if { [expr ($datapath_option & 0x3)] } {
        #  CSR Clock Source 
        add_instance csr_clk_module clock_source  
        set_instance_parameter csr_clk_module clockFrequencyKnown "false"
        
        add_interface csr_clk clock end
        set_interface_property csr_clk export_of csr_clk_module.clk_in
        
        add_interface csr_reset reset end
        set_interface_property csr_reset export_of csr_clk_module.clk_in_reset
        
        #  Merlin Master Translator
        add_instance merlin_master_translator altera_merlin_master_translator 
        set_instance_parameter merlin_master_translator AV_ADDRESS_W "13"
        set_instance_parameter merlin_master_translator AV_DATA_W "32"
        set_instance_parameter merlin_master_translator AV_BURSTCOUNT_W "1"
        set_instance_parameter merlin_master_translator AV_BYTEENABLE_W "4"
        set_instance_parameter merlin_master_translator UAV_ADDRESS_W "15"
        set_instance_parameter merlin_master_translator UAV_BURSTCOUNT_W "3"
        set_instance_parameter merlin_master_translator AV_READLATENCY "0"
        set_instance_parameter merlin_master_translator AV_WRITE_WAIT "0"
        set_instance_parameter merlin_master_translator AV_READ_WAIT "0"
        set_instance_parameter merlin_master_translator AV_DATA_HOLD "0"
        set_instance_parameter merlin_master_translator AV_SETUP_WAIT "0"
        set_instance_parameter merlin_master_translator USE_READDATA "1"
        set_instance_parameter merlin_master_translator USE_WRITEDATA "1"
        set_instance_parameter merlin_master_translator USE_READ "1"
        set_instance_parameter merlin_master_translator USE_WRITE "1"
        set_instance_parameter merlin_master_translator USE_BEGINBURSTTRANSFER "0"
        set_instance_parameter merlin_master_translator USE_BEGINTRANSFER "0"
        set_instance_parameter merlin_master_translator USE_BYTEENABLE "0"
        set_instance_parameter merlin_master_translator USE_CHIPSELECT "0"
        set_instance_parameter merlin_master_translator USE_ADDRESS "1"
        set_instance_parameter merlin_master_translator USE_BURSTCOUNT "0"
        set_instance_parameter merlin_master_translator USE_READDATAVALID "0"
        set_instance_parameter merlin_master_translator USE_WAITREQUEST "1"
        set_instance_parameter merlin_master_translator USE_LOCK "0"
        set_instance_parameter merlin_master_translator AV_SYMBOLS_PER_WORD "4"
        set_instance_parameter merlin_master_translator AV_ADDRESS_SYMBOLS "0"
        set_instance_parameter merlin_master_translator AV_BURSTCOUNT_SYMBOLS "0"
        set_instance_parameter merlin_master_translator AV_CONSTANT_BURST_BEHAVIOR "0"
        set_instance_parameter merlin_master_translator AV_LINEWRAPBURSTS "0"
        set_instance_parameter merlin_master_translator AV_MAX_PENDING_READ_TRANSACTIONS "0"
        set_instance_parameter merlin_master_translator AV_BURSTBOUNDARIES "0"
        set_instance_parameter merlin_master_translator AV_INTERLEAVEBURSTS "0"
        set_instance_parameter merlin_master_translator AV_BITS_PER_SYMBOL "8"
        set_instance_parameter merlin_master_translator AV_ISBIGENDIAN "0"
        set_instance_parameter merlin_master_translator AV_ADDRESSGROUP "0"
        set_instance_parameter merlin_master_translator UAV_ADDRESSGROUP "0"
        set_instance_parameter merlin_master_translator AV_REGISTEROUTGOINGSIGNALS "0"
        set_instance_parameter merlin_master_translator AV_REGISTERINCOMINGSIGNALS "0"
        set_instance_parameter merlin_master_translator AV_ALWAYSBURSTMAXBURST "0"
        
        add_connection csr_clk_module.clk/merlin_master_translator.clk
        add_connection csr_clk_module.clk_reset/merlin_master_translator.reset
        
        # XGMII Conversion
        add_instance interface_conversion altera_eth_interface_conversion
        set_instance_parameter interface_conversion DATAPATH_OPTION $datapath_option
        
        #  Export CSR
        add_interface csr avalon end
        set_interface_property csr export_of merlin_master_translator.avalon_anti_master_0
    }
    
    # TX
    if { [expr ($datapath_option & 0x1)] } {
        #  TX Clock Source 
        add_instance tx_clk_module clock_source  
        set_instance_parameter tx_clk_module clockFrequencyKnown "false"
        
        add_interface tx_clk clock end
        set_interface_property tx_clk export_of tx_clk_module.clk_in
        
        add_interface tx_reset reset end
        set_interface_property tx_reset export_of tx_clk_module.clk_in_reset
        
        # MM Bridge
        add_instance tx_bridge altera_avalon_mm_bridge
        set_instance_parameter tx_bridge DATA_WIDTH 32
        set_instance_parameter tx_bridge ADDRESS_WIDTH 14
        set_instance_parameter tx_bridge MAX_PENDING_RESPONSES 1
        set_instance_parameter tx_bridge PIPELINE_COMMAND 0
        set_instance_parameter tx_bridge PIPELINE_RESPONSE 0
        
        add_connection merlin_master_translator.avalon_universal_master_0 tx_bridge.s0
        set_connection_parameter merlin_master_translator.avalon_universal_master_0/tx_bridge.s0 baseAddress 0x4000
        
        # Clock Connections
        add_connection tx_clk_module.clk/tx_bridge.clk
        add_connection tx_clk_module.clk_reset/tx_bridge.reset
        add_connection tx_clk_module.clk/interface_conversion.clock_reset_tx
        add_connection tx_clk_module.clk_reset/interface_conversion.clock_reset_tx_reset
        
        # Compose MAC TX
        compose_eth_10g_mac_tx tx_clk_module.clk tx_clk_module.clk_reset tx_bridge.m0 $stat_ena $reg_stat $crc_tx $preamble_mode $pfc_ena $pfc_priority_num
        
        # XGMII Conversion
        add_connection tx_eth_link_fault_generation.mii_src/interface_conversion.xgmii_tx_bridge_sink
        
        #  Export Avalon-ST TX
        add_interface avalon_st_tx avalon_streaming end
		if { $preamble_mode == 1 } {
            add_instance tx_preamble_passthrough altera_eth_preamble_custom
            add_connection tx_clk_module.clk/tx_preamble_passthrough.clock
            add_connection tx_clk_module.clk_reset/tx_preamble_passthrough.reset
            set_interface_property avalon_st_tx export_of tx_preamble_passthrough.avalon_streaming_sink_user
            add_connection tx_preamble_passthrough.avalon_streaming_source_user/tx_eth_packet_underflow_control.avalon_streaming_sink
            add_connection tx_preamble_passthrough.preamble_src/tx_eth_packet_formatter.preamble_sink
		} else {
            set_interface_property avalon_st_tx export_of tx_eth_packet_underflow_control.avalon_streaming_sink
		}
        
        #  Export Avalon-ST Pause Generation TX
        add_interface avalon_st_pause avalon_streaming end
        set_interface_property avalon_st_pause export_of tx_eth_pause_ctrl_gen.pause_control
        
        #  Export Avalon-ST PFC TX
        if { $pfc_ena == 1 } {
            #  Export Avalon-ST PFC Generation Control TX
            add_interface avalon_st_tx_pfc_gen avalon_streaming end
            set_interface_property avalon_st_tx_pfc_gen export_of tx_eth_pfc_gen.pfc_ctrl_sink
            
            #  Export of PFC Status TX
            add_interface avalon_st_tx_pfc_status avalon_streaming start
            set_interface_property avalon_st_tx_pfc_status export_of tx_eth_pfc_gen.pfc_status_src
        }
        
        #  Export of XGMII TX
        add_interface xgmii_tx avalon_streaming start
        set_interface_property xgmii_tx export_of interface_conversion.xgmii_tx_bridge_src
		
        #  Export of TX Status
		if { $stat_ena == 1 } {
            add_interface avalon_st_txstatus avalon_streaming start
            set_interface_property avalon_st_txstatus export_of tx_st_timing_adapter_splitter_status_output.out
		}
		
    }
    
    # RX
    if { [expr ($datapath_option & 0x2)] } {
        #  RX Clock Source 
        add_instance rx_clk_module clock_source  
        set_instance_parameter rx_clk_module clockFrequencyKnown "false"
        
        add_interface rx_clk clock end
        set_interface_property rx_clk export_of rx_clk_module.clk_in
        
        add_interface rx_reset reset end
        set_interface_property rx_reset export_of rx_clk_module.clk_in_reset
        
        # MM Bridge
        add_instance rx_bridge altera_avalon_mm_bridge
        set_instance_parameter rx_bridge DATA_WIDTH 32
        set_instance_parameter rx_bridge ADDRESS_WIDTH 14
        set_instance_parameter rx_bridge MAX_PENDING_RESPONSES 1
        set_instance_parameter rx_bridge PIPELINE_COMMAND 0
        set_instance_parameter rx_bridge PIPELINE_RESPONSE 0
        
        add_connection merlin_master_translator.avalon_universal_master_0 rx_bridge.s0
        set_connection_parameter merlin_master_translator.avalon_universal_master_0/rx_bridge.s0 baseAddress 0x0
        
        # Clock Connections
        add_connection rx_clk_module.clk/rx_bridge.clk
        add_connection rx_clk_module.clk_reset/rx_bridge.reset
        add_connection rx_clk_module.clk/interface_conversion.clock_reset_rx
        add_connection rx_clk_module.clk_reset/interface_conversion.clock_reset_rx_reset
        
        # Compose MAC RX
        compose_eth_10g_mac_rx rx_clk_module.clk rx_clk_module.clk_reset rx_bridge.m0 $stat_ena $reg_stat $supp_addr_ena $preamble_mode $pfc_ena $pfc_priority_num
        
        # XGMII Conversion
        add_connection interface_conversion.xgmii_rx_bridge_src/rx_st_timing_adapter_interface_conversion.in
        
        #  Export of XGMII RX
        add_interface xgmii_rx avalon_streaming end
        set_interface_property xgmii_rx export_of interface_conversion.xgmii_rx_bridge_sink
        
        #  Export Avalon-ST RX
        add_interface avalon_st_rx avalon_streaming start
        set_interface_property avalon_st_rx export_of rx_eth_packet_overflow_control.avalon_streaming_source
        
        #  Export of RX Status
        add_interface avalon_st_rxstatus avalon_streaming start
        if { $preamble_mode == 1 } {
			set_interface_property avalon_st_rxstatus export_of rx_st_status_output_delay_final.out
		} else {
			set_interface_property avalon_st_rxstatus export_of rx_st_status_output_delay.out
		}
        
        #  Export Avalon-ST PFC RX
        if { $pfc_ena == 1 } {
            #  Export of PFC Pause Enable RX
            add_interface avalon_st_rx_pfc_pause avalon_streaming start
            set_interface_property avalon_st_rx_pfc_pause export_of rx_eth_pfc_pause_conversion.pfc_pause_ena_src
            
            #  Export of PFC Status RX
            add_interface avalon_st_rx_pfc_status avalon_streaming start
            set_interface_property avalon_st_rx_pfc_status export_of rx_eth_delay_pfc_status.out
        }
    }
    
    # TX Only
    if { $datapath_option == 1 } {
        #  Export of Pause Length
        add_interface avalon_st_tx_pause_length avalon_streaming end
        set_interface_property avalon_st_tx_pause_length export_of tx_eth_pause_beat_conversion.pause_quanta_sink
        
        #  Export of Link Fault Status
        add_interface link_fault_status_xgmii_tx avalon_streaming end
        set_interface_property link_fault_status_xgmii_tx export_of tx_eth_link_fault_generation.link_fault_sink
    }
    
    # RX Only
    if { $datapath_option == 2 } {
        #  Export of Pause Length
        add_interface avalon_st_rx_pause_length avalon_streaming start
        set_interface_property avalon_st_rx_pause_length export_of rx_eth_frame_status_merger.pauselen_src
        
        #  Export of Link Fault Status
        add_interface link_fault_status_xgmii_rx avalon_streaming start
        set_interface_property link_fault_status_xgmii_rx export_of rx_eth_link_fault_detection.link_fault_src
    }
    
    # TX & RX
    if { $datapath_option == 3 } {
        # Crossing between RX and TX
        # Link Fault Signaling
        #  Timing Adapter Link Fault Status RX
        add_instance txrx_timing_adapter_link_fault_status_rx timing_adapter
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx generationLanguage "VERILOG"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inBitsPerSymbol "2"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inChannelWidth "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inErrorDescriptor ""
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inErrorWidth "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inMaxChannel "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inReadyLatency "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inSymbolsPerBeat "1"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inUseEmpty "true"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inUseEmptyPort "YES"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inUsePackets "false"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inUseReady "false"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx inUseValid "false"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx moduleName ""
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx outReadyLatency "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx outUseReady "true"
        set_instance_parameter txrx_timing_adapter_link_fault_status_rx outUseValid "true"
        
        #  Avalon St Splitter Link Fault Status RX
        add_instance txrx_st_splitter_link_fault_status altera_avalon_st_splitter
        set_instance_parameter txrx_st_splitter_link_fault_status NUMBER_OF_OUTPUTS "2"
        set_instance_parameter txrx_st_splitter_link_fault_status QUALIFY_VALID_OUT "0"
        set_instance_parameter txrx_st_splitter_link_fault_status DATA_WIDTH "2"
        set_instance_parameter txrx_st_splitter_link_fault_status BITS_PER_SYMBOL "2"
        set_instance_parameter txrx_st_splitter_link_fault_status USE_PACKETS "0"
        set_instance_parameter txrx_st_splitter_link_fault_status USE_CHANNEL "0"
        set_instance_parameter txrx_st_splitter_link_fault_status CHANNEL_WIDTH "1"
        set_instance_parameter txrx_st_splitter_link_fault_status MAX_CHANNELS "1"
        set_instance_parameter txrx_st_splitter_link_fault_status USE_ERROR "0"
        set_instance_parameter txrx_st_splitter_link_fault_status ERROR_WIDTH "1"
        
        #  Timing Adapter Link Fault Status RX Export
        add_instance txrx_timing_adapter_link_fault_status_export timing_adapter
        set_instance_parameter txrx_timing_adapter_link_fault_status_export generationLanguage "VERILOG"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inBitsPerSymbol "2"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inChannelWidth "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inErrorDescriptor ""
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inErrorWidth "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inMaxChannel "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inReadyLatency "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inSymbolsPerBeat "1"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inUseEmpty "true"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inUseEmptyPort "YES"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inUsePackets "false"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inUseReady "true"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export inUseValid "true"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export moduleName ""
        set_instance_parameter txrx_timing_adapter_link_fault_status_export outReadyLatency "0"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export outUseReady "false"
        set_instance_parameter txrx_timing_adapter_link_fault_status_export outUseValid "false"
        
        # DC Fifo ( link_fault_src to link_fault_sink )
        add_instance rxtx_dc_fifo_link_fault_status altera_avalon_dc_fifo
        set_instance_parameter rxtx_dc_fifo_link_fault_status SYMBOLS_PER_BEAT "1"
        set_instance_parameter rxtx_dc_fifo_link_fault_status BITS_PER_SYMBOL "2"
        set_instance_parameter rxtx_dc_fifo_link_fault_status FIFO_DEPTH "16"
        set_instance_parameter rxtx_dc_fifo_link_fault_status CHANNEL_WIDTH "0"
        set_instance_parameter rxtx_dc_fifo_link_fault_status ERROR_WIDTH "0"
        set_instance_parameter rxtx_dc_fifo_link_fault_status USE_PACKETS "0"
        set_instance_parameter rxtx_dc_fifo_link_fault_status USE_IN_FILL_LEVEL "0"
        set_instance_parameter rxtx_dc_fifo_link_fault_status USE_OUT_FILL_LEVEL "0"
        set_instance_parameter rxtx_dc_fifo_link_fault_status WR_SYNC_DEPTH "2"
        set_instance_parameter rxtx_dc_fifo_link_fault_status RD_SYNC_DEPTH "2"
        set_instance_parameter rxtx_dc_fifo_link_fault_status ENABLE_EXPLICIT_MAXCHANNEL "false"
        set_instance_parameter rxtx_dc_fifo_link_fault_status EXPLICIT_MAXCHANNEL "0"
        
        #  Timing Adapter Link Fault TX
        add_instance rxtx_timing_adapter_link_fault_status_tx timing_adapter
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx generationLanguage "VERILOG"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inBitsPerSymbol "2"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inChannelWidth "0"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inErrorDescriptor ""
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inErrorWidth "0"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inMaxChannel "0"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inReadyLatency "0"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inSymbolsPerBeat "1"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inUseEmpty "true"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inUseEmptyPort "YES"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inUsePackets "false"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inUseReady "true"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx inUseValid "true"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx moduleName ""
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx outReadyLatency "0"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx outUseReady "false"
        set_instance_parameter rxtx_timing_adapter_link_fault_status_tx outUseValid "false"
        
        # Clock Connections
        add_connection rx_clk_module.clk/txrx_timing_adapter_link_fault_status_rx.clk
        add_connection rx_clk_module.clk_reset/txrx_timing_adapter_link_fault_status_rx.reset
        add_connection rx_clk_module.clk/txrx_st_splitter_link_fault_status.clk
        #add_connection rx_clk_module.clk_reset/txrx_st_splitter_link_fault_status.clk
        add_connection rx_clk_module.clk/txrx_timing_adapter_link_fault_status_export.clk
        add_connection rx_clk_module.clk_reset/txrx_timing_adapter_link_fault_status_export.reset
        add_connection rx_clk_module.clk/rxtx_dc_fifo_link_fault_status.in_clk
        add_connection rx_clk_module.clk_reset/rxtx_dc_fifo_link_fault_status.in_clk_reset
        
        add_connection tx_clk_module.clk/rxtx_dc_fifo_link_fault_status.out_clk
        add_connection tx_clk_module.clk_reset/rxtx_dc_fifo_link_fault_status.out_clk_reset
        add_connection tx_clk_module.clk/rxtx_timing_adapter_link_fault_status_tx.clk
        add_connection tx_clk_module.clk_reset/rxtx_timing_adapter_link_fault_status_tx.reset
        
        # Connections
        add_connection rx_eth_link_fault_detection.link_fault_src/txrx_timing_adapter_link_fault_status_rx.in
        add_connection txrx_timing_adapter_link_fault_status_rx.out/txrx_st_splitter_link_fault_status.in
        add_connection txrx_st_splitter_link_fault_status.out0/txrx_timing_adapter_link_fault_status_export.in
        
        add_connection txrx_st_splitter_link_fault_status.out1/rxtx_dc_fifo_link_fault_status.in
        add_connection rxtx_dc_fifo_link_fault_status.out/rxtx_timing_adapter_link_fault_status_tx.in
        add_connection rxtx_timing_adapter_link_fault_status_tx.out/tx_eth_link_fault_generation.link_fault_sink
        
        #  Export of Link Fault Status
        add_interface link_fault_status_xgmii_rx avalon_streaming start
        set_interface_property link_fault_status_xgmii_rx export_of txrx_timing_adapter_link_fault_status_export.out
        
        
        
        # Pause Length
        #  Timing Adapter Pauselen RX
        add_instance rxtx_timing_adapter_pauselen_rx timing_adapter
        set_instance_parameter rxtx_timing_adapter_pauselen_rx generationLanguage "VERILOG"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inBitsPerSymbol "16"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inChannelWidth "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inErrorDescriptor ""
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inErrorWidth "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inMaxChannel "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inReadyLatency "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inSymbolsPerBeat "1"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inUseEmpty "true"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inUseEmptyPort "YES"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inUsePackets "false"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inUseReady "false"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx inUseValid "true"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx moduleName ""
        set_instance_parameter rxtx_timing_adapter_pauselen_rx outReadyLatency "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx outUseReady "true"
        set_instance_parameter rxtx_timing_adapter_pauselen_rx outUseValid "true"
        
        # DC Fifo ( frame decoder to pkt backpressure )
        add_instance rxtx_dc_fifo_pauselen altera_avalon_dc_fifo
        set_instance_parameter rxtx_dc_fifo_pauselen SYMBOLS_PER_BEAT "1"
        set_instance_parameter rxtx_dc_fifo_pauselen BITS_PER_SYMBOL "16"
        set_instance_parameter rxtx_dc_fifo_pauselen FIFO_DEPTH "16"
        set_instance_parameter rxtx_dc_fifo_pauselen CHANNEL_WIDTH "0"
        set_instance_parameter rxtx_dc_fifo_pauselen ERROR_WIDTH "0"
        set_instance_parameter rxtx_dc_fifo_pauselen USE_PACKETS "0"
        set_instance_parameter rxtx_dc_fifo_pauselen USE_IN_FILL_LEVEL "0"
        set_instance_parameter rxtx_dc_fifo_pauselen USE_OUT_FILL_LEVEL "0"
        set_instance_parameter rxtx_dc_fifo_pauselen WR_SYNC_DEPTH "2"
        set_instance_parameter rxtx_dc_fifo_pauselen RD_SYNC_DEPTH "2"
        set_instance_parameter rxtx_dc_fifo_pauselen ENABLE_EXPLICIT_MAXCHANNEL "false"
        set_instance_parameter rxtx_dc_fifo_pauselen EXPLICIT_MAXCHANNEL "0"
        
        #  Timing Adapter Pauselen TX
        add_instance rxtx_timing_adapter_pauselen_tx timing_adapter
        set_instance_parameter rxtx_timing_adapter_pauselen_tx generationLanguage "VERILOG"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inBitsPerSymbol "16"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inChannelWidth "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inErrorDescriptor ""
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inErrorWidth "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inMaxChannel "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inReadyLatency "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inSymbolsPerBeat "1"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inUseEmpty "true"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inUseEmptyPort "YES"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inUsePackets "false"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inUseReady "true"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx inUseValid "true"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx moduleName ""
        set_instance_parameter rxtx_timing_adapter_pauselen_tx outReadyLatency "0"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx outUseReady "false"
        set_instance_parameter rxtx_timing_adapter_pauselen_tx outUseValid "true"
        
        # Clock Connections
        add_connection rx_clk_module.clk/rxtx_dc_fifo_pauselen.in_clk
        add_connection rx_clk_module.clk_reset/rxtx_dc_fifo_pauselen.in_clk_reset
        add_connection tx_clk_module.clk/rxtx_dc_fifo_pauselen.out_clk
        add_connection tx_clk_module.clk_reset/rxtx_dc_fifo_pauselen.out_clk_reset
        add_connection rx_clk_module.clk/rxtx_timing_adapter_pauselen_rx.clk
        add_connection rx_clk_module.clk_reset/rxtx_timing_adapter_pauselen_rx.reset
        add_connection tx_clk_module.clk/rxtx_timing_adapter_pauselen_tx.clk
        add_connection tx_clk_module.clk_reset/rxtx_timing_adapter_pauselen_tx.reset
        
        # Connections
        add_connection rx_eth_frame_status_merger.pauselen_src/rxtx_timing_adapter_pauselen_rx.in
        add_connection rxtx_timing_adapter_pauselen_rx.out/rxtx_dc_fifo_pauselen.in
        add_connection rxtx_dc_fifo_pauselen.out/rxtx_timing_adapter_pauselen_tx.in
        add_connection rxtx_timing_adapter_pauselen_tx.out/tx_eth_pause_beat_conversion.pause_quanta_sink
    }
    
}

proc compose_eth_10g_mac_rx {rx_clock_src rx_reset_src rx_csr_master stat_ena reg_stat supp_addr_ena preamble_mode pfc_ena pfc_priority_num} {
    compose_eth_xgmii_rx $rx_clock_src $rx_reset_src $preamble_mode
    compose_eth_mac_rx $rx_clock_src $rx_reset_src $rx_csr_master $stat_ena $reg_stat $supp_addr_ena $preamble_mode $pfc_ena $pfc_priority_num
    
    add_connection rx_eth_lane_decoder.avalon_streaming_source/rx_eth_pkt_backpressure_control.avalon_st_sink_data
}

proc compose_eth_10g_mac_tx {tx_clock_src tx_reset_src tx_csr_master stat_ena reg_stat crc_tx preamble_mode pfc_ena pfc_priority_num} {
    compose_eth_mac_tx $tx_clock_src $tx_reset_src $tx_csr_master $stat_ena $reg_stat $crc_tx $preamble_mode $pfc_ena $pfc_priority_num
    compose_eth_xgmii_tx $tx_clock_src $tx_reset_src $preamble_mode
	
}

proc compose_eth_mac_rx {rx_clock_src rx_reset_src rx_csr_master stat_ena reg_stat supp_addr_ena preamble_mode pfc_ena pfc_priority_num} {
    #  Packet Back Pressure RX
    add_instance rx_eth_pkt_backpressure_control altera_eth_pkt_backpressure_control
    set_instance_parameter rx_eth_pkt_backpressure_control BITSPERSYMBOL "8"
    set_instance_parameter rx_eth_pkt_backpressure_control SYMBOLSPERBEAT "8"
    set_instance_parameter rx_eth_pkt_backpressure_control ERROR_WIDTH "1"
    set_instance_parameter rx_eth_pkt_backpressure_control USE_READY "0"
    set_instance_parameter rx_eth_pkt_backpressure_control USE_PAUSE "0"
    
    #  Timing Adapter Frame Status In RX
    add_instance rx_st_timing_adapter_frame_status_in timing_adapter
    set_instance_parameter rx_st_timing_adapter_frame_status_in generationLanguage "VERILOG"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inBitsPerSymbol "8"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inChannelWidth "0"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inErrorDescriptor ""
    set_instance_parameter rx_st_timing_adapter_frame_status_in inErrorWidth "1"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inMaxChannel "0"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inSymbolsPerBeat "8"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inUseEmpty "true"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inUseEmptyPort "YES"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inUsePackets "true"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inUseReady "false"
    set_instance_parameter rx_st_timing_adapter_frame_status_in inUseValid "true"
    set_instance_parameter rx_st_timing_adapter_frame_status_in moduleName ""
    set_instance_parameter rx_st_timing_adapter_frame_status_in outReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_frame_status_in outUseReady "true"
    set_instance_parameter rx_st_timing_adapter_frame_status_in outUseValid "true"
    
    # Connections
    add_connection rx_eth_pkt_backpressure_control.avalon_st_source_data/rx_st_timing_adapter_frame_status_in.in
    
    #  Avalon St Splitter RX
    add_instance rx_st_frame_status_splitter altera_avalon_st_splitter
    set_instance_parameter rx_st_frame_status_splitter NUMBER_OF_OUTPUTS "2"
    set_instance_parameter rx_st_frame_status_splitter QUALIFY_VALID_OUT "1"
    set_instance_parameter rx_st_frame_status_splitter DATA_WIDTH "64"
    set_instance_parameter rx_st_frame_status_splitter BITS_PER_SYMBOL "8"
    set_instance_parameter rx_st_frame_status_splitter USE_PACKETS "1"
    set_instance_parameter rx_st_frame_status_splitter USE_CHANNEL "0"
    set_instance_parameter rx_st_frame_status_splitter CHANNEL_WIDTH "1"
    set_instance_parameter rx_st_frame_status_splitter MAX_CHANNELS "1"
    set_instance_parameter rx_st_frame_status_splitter USE_ERROR "1"
    set_instance_parameter rx_st_frame_status_splitter ERROR_WIDTH "1"
    
    #  Frame Decoder RX
    add_instance rx_eth_frame_decoder altera_eth_frame_decoder
    set_instance_parameter rx_eth_frame_decoder BITSPERSYMBOL "8"
    set_instance_parameter rx_eth_frame_decoder SYMBOLSPERBEAT "8"
    set_instance_parameter rx_eth_frame_decoder ERROR_WIDTH "1"
    set_instance_parameter rx_eth_frame_decoder ENABLE_SUPP_ADDR $supp_addr_ena
    set_instance_parameter rx_eth_frame_decoder ENABLE_DATA_SOURCE "1"
    set_instance_parameter rx_eth_frame_decoder ENABLE_PAUSELEN "1"
    set_instance_parameter rx_eth_frame_decoder ENABLE_PKTINFO "1"
    set_instance_parameter rx_eth_frame_decoder USE_READY "0"
    set_instance_parameter rx_eth_frame_decoder ENABLE_PFC $pfc_ena
    set_instance_parameter rx_eth_frame_decoder PFC_PRIORITY_NUM $pfc_priority_num
    set_instance_parameter rx_eth_frame_decoder ENABLE_PFC_PQ "1"
    set_instance_parameter rx_eth_frame_decoder ENABLE_PFC_STATUS "1"
    
    #  CRC Checker Rx
    add_instance rx_eth_crc_checker altera_eth_crc
    set_instance_parameter rx_eth_crc_checker BITSPERSYMBOL "8"
    set_instance_parameter rx_eth_crc_checker SYMBOLSPERBEAT "8"
    set_instance_parameter rx_eth_crc_checker ERROR_WIDTH "1"
    set_instance_parameter rx_eth_crc_checker MODE_CHECKER_0_INSERTER_1 "0"
    set_instance_parameter rx_eth_crc_checker USE_READY "false"
    set_instance_parameter rx_eth_crc_checker USE_CHANNEL "0"
    
    #  Timing Adapter Frame Status Out Frame Decoder RX
    add_instance rx_timing_adapter_frame_status_out_frame_decoder timing_adapter
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder generationLanguage "VERILOG"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inBitsPerSymbol "8"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inChannelWidth "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inErrorDescriptor ""
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inErrorWidth "1"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inMaxChannel "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inReadyLatency "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inSymbolsPerBeat "8"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inUseEmpty "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inUseEmptyPort "YES"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inUsePackets "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inUseReady "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder inUseValid "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder moduleName ""
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder outReadyLatency "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder outUseReady "false"
    set_instance_parameter rx_timing_adapter_frame_status_out_frame_decoder outUseValid "true"
    
    #  Timing Adapter Frame Status Out CRC Checker RX
    add_instance rx_timing_adapter_frame_status_out_crc_checker timing_adapter
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker generationLanguage "VERILOG"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inBitsPerSymbol "8"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inChannelWidth "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inErrorDescriptor ""
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inErrorWidth "1"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inMaxChannel "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inReadyLatency "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inSymbolsPerBeat "8"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inUseEmpty "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inUseEmptyPort "YES"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inUsePackets "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inUseReady "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker inUseValid "true"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker moduleName ""
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker outReadyLatency "0"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker outUseReady "false"
    set_instance_parameter rx_timing_adapter_frame_status_out_crc_checker outUseValid "true"
    
    # Frame Status Merger RX
    add_instance rx_eth_frame_status_merger altera_eth_frame_status_merger
    set_instance_parameter rx_eth_frame_status_merger ENABLE_PFC $pfc_ena
    set_instance_parameter rx_eth_frame_status_merger PFC_PRIORITY_NUM $pfc_priority_num
    
    # Connections
    add_connection rx_st_timing_adapter_frame_status_in.out/rx_st_frame_status_splitter.in
    add_connection rx_st_frame_status_splitter.out0/rx_timing_adapter_frame_status_out_frame_decoder.in
    add_connection rx_timing_adapter_frame_status_out_frame_decoder.out/rx_eth_frame_decoder.avalon_st_data_sink
    add_connection rx_eth_frame_decoder.avalon_st_data_src/rx_eth_frame_status_merger.frame_decoder_data_sink
    add_connection rx_eth_frame_decoder.avalon_st_pauselen_src/rx_eth_frame_status_merger.pauselen_sink
    add_connection rx_st_frame_status_splitter.out1/rx_timing_adapter_frame_status_out_crc_checker.in
    add_connection rx_timing_adapter_frame_status_out_crc_checker.out/rx_eth_crc_checker.avalon_streaming_sink
    
    # CRC Pad Remover RX
    add_instance rx_eth_crc_pad_rem altera_eth_crc_pad_rem
    set_instance_parameter rx_eth_crc_pad_rem BITSPERSYMBOL "8"
    set_instance_parameter rx_eth_crc_pad_rem SYMBOLSPERBEAT "8"
    set_instance_parameter rx_eth_crc_pad_rem ERRORWIDTH "5"
    
    # Connections
    add_connection rx_eth_frame_status_merger.data_src/rx_eth_crc_pad_rem.avalon_streaming_sink_data
    add_connection rx_eth_frame_decoder.avalon_st_pktinfo_src/rx_eth_crc_pad_rem.avalon_streaming_sink_status
    
	if {$preamble_mode == 1} {
		# Preamble Inserter on RX
		add_instance rx_preamble_inserter altera_eth_preamble_inserter
		
		# Connections
		add_connection rx_eth_lane_decoder.preamble_src/rx_preamble_inserter.preamble_sink
		add_connection rx_eth_crc_pad_rem.avalon_streaming_source_data/rx_preamble_inserter.avalon_streaming_sink_data_line
	}
	
    #  Overflow Controller RX
    add_instance rx_eth_packet_overflow_control altera_eth_packet_overflow_control
    set_instance_parameter rx_eth_packet_overflow_control BITSPERSYMBOL "8"
    set_instance_parameter rx_eth_packet_overflow_control SYMBOLSPERBEAT "8"
    set_instance_parameter rx_eth_packet_overflow_control ERROR_WIDTH "5"
    
    # Connections
    if {$preamble_mode == 0} {
		add_connection rx_eth_crc_pad_rem.avalon_streaming_source_data/rx_eth_packet_overflow_control.avalon_streaming_sink
    } else {
		add_connection rx_preamble_inserter.avalon_streaming_source_data_line/rx_eth_packet_overflow_control.avalon_streaming_sink
	}
    # Avalon ST Delay to align avalon_st_rx with avalon_st_rxstatus
    add_instance rx_st_status_output_delay altera_avalon_st_delay
    set_instance_parameter rx_st_status_output_delay NUMBER_OF_DELAY_CLOCKS "2"
    set_instance_parameter rx_st_status_output_delay DATA_WIDTH "40"
    set_instance_parameter rx_st_status_output_delay BITS_PER_SYMBOL "40"
    set_instance_parameter rx_st_status_output_delay USE_PACKETS "0"
    set_instance_parameter rx_st_status_output_delay USE_CHANNEL "0"
    set_instance_parameter rx_st_status_output_delay CHANNEL_WIDTH "1"
    set_instance_parameter rx_st_status_output_delay MAX_CHANNELS "1"
    set_instance_parameter rx_st_status_output_delay USE_ERROR "1"
    set_instance_parameter rx_st_status_output_delay ERROR_WIDTH "7"
    
    if { $pfc_ena == 1} {
        # PFC Pause Quanta Conversion
        add_instance rx_eth_pfc_pause_conversion altera_eth_pfc_pause_conversion
        set_instance_parameter rx_eth_pfc_pause_conversion PFC_PRIORITY_NUM $pfc_priority_num
        
        add_connection rx_eth_frame_decoder.avalon_st_pfc_pause_quanta_src/rx_eth_frame_status_merger.pfc_pause_quanta_sink
        add_connection rx_eth_frame_status_merger.pfc_pause_quanta_src/rx_eth_pfc_pause_conversion.pfc_pause_quanta_sink
        
        # Avalon ST Delay to align output of RX Frame Decoder and CRC Checker
        add_instance rx_st_delay_pfc_crc altera_avalon_st_delay
        set_instance_parameter rx_st_delay_pfc_crc NUMBER_OF_DELAY_CLOCKS "2"
        set_instance_parameter rx_st_delay_pfc_crc DATA_WIDTH "64"
        set_instance_parameter rx_st_delay_pfc_crc BITS_PER_SYMBOL "8"
        set_instance_parameter rx_st_delay_pfc_crc USE_PACKETS "1"
        set_instance_parameter rx_st_delay_pfc_crc USE_CHANNEL "0"
        set_instance_parameter rx_st_delay_pfc_crc CHANNEL_WIDTH "1"
        set_instance_parameter rx_st_delay_pfc_crc MAX_CHANNELS "1"
        set_instance_parameter rx_st_delay_pfc_crc USE_ERROR "1"
        set_instance_parameter rx_st_delay_pfc_crc ERROR_WIDTH "2"
        
        add_connection rx_eth_crc_checker.avalon_streaming_source/rx_st_delay_pfc_crc.in
        add_connection rx_st_delay_pfc_crc.out/rx_eth_frame_status_merger.crc_checker_data_sink
        
        # Avalon ST Delay to align avalon_st_rx with avalon_st_rx_pfc_status
        add_instance rx_eth_delay_pfc_status altera_avalon_st_delay
        set_instance_parameter rx_eth_delay_pfc_status NUMBER_OF_DELAY_CLOCKS "2"
        set_instance_parameter rx_eth_delay_pfc_status DATA_WIDTH [ expr 2 * $pfc_priority_num ]
        set_instance_parameter rx_eth_delay_pfc_status BITS_PER_SYMBOL [ expr 2 * $pfc_priority_num ]
        set_instance_parameter rx_eth_delay_pfc_status USE_PACKETS "0"
        set_instance_parameter rx_eth_delay_pfc_status USE_CHANNEL "0"
        set_instance_parameter rx_eth_delay_pfc_status CHANNEL_WIDTH "1"
        set_instance_parameter rx_eth_delay_pfc_status MAX_CHANNELS "1"
        set_instance_parameter rx_eth_delay_pfc_status USE_ERROR "0"
        set_instance_parameter rx_eth_delay_pfc_status ERROR_WIDTH "1"
        
        if { $preamble_mode == 1} {
            set_instance_parameter rx_eth_delay_pfc_status NUMBER_OF_DELAY_CLOCKS "3"
        } else {
            set_instance_parameter rx_eth_delay_pfc_status NUMBER_OF_DELAY_CLOCKS "2"
        }
        
        add_connection rx_eth_frame_decoder.avalon_st_pfc_status_src/rx_eth_delay_pfc_status.in
    } else {
        add_connection rx_eth_crc_checker.avalon_streaming_source/rx_eth_frame_status_merger.crc_checker_data_sink
    }
    
    # RX Status Connections
    add_connection rx_eth_frame_decoder.avalon_st_rxstatus_src/rx_eth_frame_status_merger.rxstatus_sink
    
	if {$preamble_mode == 1} {
	
		# Splitter for RX Status After First Delay
		add_instance rx_st_status_splitter_2 altera_avalon_st_splitter 
		set_instance_parameter rx_st_status_splitter_2 NUMBER_OF_OUTPUTS "2"
		set_instance_parameter rx_st_status_splitter_2 QUALIFY_VALID_OUT "1"
		set_instance_parameter rx_st_status_splitter_2 DATA_WIDTH "40"
		set_instance_parameter rx_st_status_splitter_2 BITS_PER_SYMBOL "40"
		set_instance_parameter rx_st_status_splitter_2 USE_PACKETS "0"
		set_instance_parameter rx_st_status_splitter_2 USE_READY "0"
		set_instance_parameter rx_st_status_splitter_2 USE_CHANNEL "0"
		set_instance_parameter rx_st_status_splitter_2 CHANNEL_WIDTH "1"
		set_instance_parameter rx_st_status_splitter_2 MAX_CHANNELS "1"
		set_instance_parameter rx_st_status_splitter_2 USE_ERROR "1"
		set_instance_parameter rx_st_status_splitter_2 ERROR_WIDTH "7"
		
		# Connections
		add_connection rx_st_status_output_delay.out/rx_st_status_splitter_2.in
		
		# Avalon ST Delay to align avalon_st_rx with avalon_st_rxstatus after Preamble Inserterion
		add_instance rx_st_status_output_delay_final altera_avalon_st_delay
		set_instance_parameter rx_st_status_output_delay_final NUMBER_OF_DELAY_CLOCKS "1"
		set_instance_parameter rx_st_status_output_delay_final DATA_WIDTH "40"
		set_instance_parameter rx_st_status_output_delay_final BITS_PER_SYMBOL "40"
		set_instance_parameter rx_st_status_output_delay_final USE_PACKETS "0"
		set_instance_parameter rx_st_status_output_delay_final USE_CHANNEL "0"
		set_instance_parameter rx_st_status_output_delay_final CHANNEL_WIDTH "1"
		set_instance_parameter rx_st_status_output_delay_final MAX_CHANNELS "1"
		set_instance_parameter rx_st_status_output_delay_final USE_ERROR "1"
		set_instance_parameter rx_st_status_output_delay_final ERROR_WIDTH "7"
		
		#Connections
		add_connection rx_st_status_splitter_2.out0/rx_preamble_inserter.avalon_streaming_sink_packet_status
		add_connection rx_st_status_splitter_2.out1/rx_st_status_output_delay_final.in
	
	}
    
    #  Error Adapter for RX Status
    add_instance rx_st_error_adapter_stat error_adapter
    set_instance_parameter rx_st_error_adapter_stat generationLanguage "VERILOG"
    set_instance_parameter rx_st_error_adapter_stat inBitsPerSymbol "40"
    set_instance_parameter rx_st_error_adapter_stat inChannelWidth "0"
    set_instance_parameter rx_st_error_adapter_stat inErrorDescriptor "payload_length,oversize,undersize,crc,phy"
    set_instance_parameter rx_st_error_adapter_stat inErrorWidth "5"
    set_instance_parameter rx_st_error_adapter_stat inMaxChannel "0"
    set_instance_parameter rx_st_error_adapter_stat inReadyLatency "0"
    set_instance_parameter rx_st_error_adapter_stat inSymbolsPerBeat "1"
    set_instance_parameter rx_st_error_adapter_stat inUseEmpty "false"
    set_instance_parameter rx_st_error_adapter_stat inUseEmptyPort "AUTO"
    set_instance_parameter rx_st_error_adapter_stat inUsePackets "false"
    set_instance_parameter rx_st_error_adapter_stat inUseReady "false"
    set_instance_parameter rx_st_error_adapter_stat moduleName ""
    set_instance_parameter rx_st_error_adapter_stat outErrorDescriptor "phy,user,underflow,crc,payload_length,oversize,undersize"
    set_instance_parameter rx_st_error_adapter_stat outErrorWidth "7"
    
    add_connection rx_eth_frame_status_merger.rxstatus_src/rx_st_error_adapter_stat.in
    
    if {$stat_ena == 1} {
        
        #  Timing Adapter for Status Splitter Input
        add_instance rx_st_timing_adapter_splitter_status_in timing_adapter
        set_instance_parameter rx_st_timing_adapter_splitter_status_in generationLanguage "VERILOG"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inBitsPerSymbol "40"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inChannelWidth "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inErrorDescriptor ""
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inErrorWidth "7"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inMaxChannel "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inReadyLatency "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inSymbolsPerBeat "1"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inUseEmpty "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inUseEmptyPort "YES"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inUsePackets "false"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inUseReady "false"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in inUseValid "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in moduleName ""
        set_instance_parameter rx_st_timing_adapter_splitter_status_in outReadyLatency "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in outUseReady "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_in outUseValid "true"
        
        # Splitter for RX Status
        add_instance rx_st_status_splitter altera_avalon_st_splitter 
        set_instance_parameter rx_st_status_splitter NUMBER_OF_OUTPUTS "2"
        set_instance_parameter rx_st_status_splitter QUALIFY_VALID_OUT "1"
        set_instance_parameter rx_st_status_splitter DATA_WIDTH "40"
        set_instance_parameter rx_st_status_splitter BITS_PER_SYMBOL "40"
        set_instance_parameter rx_st_status_splitter USE_PACKETS "0"
        set_instance_parameter rx_st_status_splitter USE_CHANNEL "0"
        set_instance_parameter rx_st_status_splitter CHANNEL_WIDTH "1"
        set_instance_parameter rx_st_status_splitter MAX_CHANNELS "1"
        set_instance_parameter rx_st_status_splitter USE_ERROR "1"
        set_instance_parameter rx_st_status_splitter ERROR_WIDTH "7"
        
        #  Timing Adapter for Statistics of Status Splitter
        add_instance rx_st_timing_adapter_splitter_status_statistics timing_adapter
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics generationLanguage "VERILOG"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inBitsPerSymbol "40"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inChannelWidth "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inErrorDescriptor ""
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inErrorWidth "7"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inMaxChannel "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inReadyLatency "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inSymbolsPerBeat "1"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inUseEmpty "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inUseEmptyPort "YES"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inUsePackets "false"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inUseReady "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics inUseValid "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics moduleName ""
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics outReadyLatency "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics outUseReady "false"
        set_instance_parameter rx_st_timing_adapter_splitter_status_statistics outUseValid "true"

        #  Timing Adapter for Statistics of Status Splitter
        add_instance rx_st_timing_adapter_splitter_status_output timing_adapter
        set_instance_parameter rx_st_timing_adapter_splitter_status_output generationLanguage "VERILOG"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inBitsPerSymbol "40"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inChannelWidth "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inErrorDescriptor ""
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inErrorWidth "7"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inMaxChannel "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inReadyLatency "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inSymbolsPerBeat "1"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inUseEmpty "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inUseEmptyPort "YES"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inUsePackets "false"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inUseReady "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output inUseValid "true"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output moduleName ""
        set_instance_parameter rx_st_timing_adapter_splitter_status_output outReadyLatency "0"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output outUseReady "false"
        set_instance_parameter rx_st_timing_adapter_splitter_status_output outUseValid "true"
        
        # Avalon ST Delay to pipeline input to RX statistics
        add_instance rx_st_status_statistics_delay altera_avalon_st_delay
        set_instance_parameter rx_st_status_statistics_delay NUMBER_OF_DELAY_CLOCKS "1"
        set_instance_parameter rx_st_status_statistics_delay DATA_WIDTH "40"
        set_instance_parameter rx_st_status_statistics_delay BITS_PER_SYMBOL "40"
        set_instance_parameter rx_st_status_statistics_delay USE_PACKETS "0"
        set_instance_parameter rx_st_status_statistics_delay USE_CHANNEL "0"
        set_instance_parameter rx_st_status_statistics_delay CHANNEL_WIDTH "1"
        set_instance_parameter rx_st_status_statistics_delay MAX_CHANNELS "1"
        set_instance_parameter rx_st_status_statistics_delay USE_ERROR "1"
        set_instance_parameter rx_st_status_statistics_delay ERROR_WIDTH "7"
        
        if {$reg_stat == 1} {
            #  Statistics Collector TX Register-Based
            add_instance rx_eth_statistics_collector altera_eth_statistics_collector
            set_instance_parameter rx_eth_statistics_collector ENABLE_PFC $pfc_ena
        } else {
            # Statistics Collector TX Memory-Based
            add_instance rx_eth_statistics_collector altera_eth_10gmem_statistics_collector
            set_instance_parameter rx_eth_statistics_collector ENABLE_PFC $pfc_ena
        }
        
        # Connections
        add_connection rx_st_error_adapter_stat.out/rx_st_timing_adapter_splitter_status_in.in
        add_connection rx_st_timing_adapter_splitter_status_in.out/rx_st_status_splitter.in
        add_connection rx_st_status_splitter.out0/rx_st_timing_adapter_splitter_status_statistics.in
        add_connection rx_st_timing_adapter_splitter_status_statistics.out/rx_st_status_statistics_delay.in
        add_connection rx_st_status_statistics_delay.out/rx_eth_statistics_collector.avalon_st_sink_data
        
        add_connection rx_st_status_splitter.out1/rx_st_timing_adapter_splitter_status_output.in
        add_connection rx_st_timing_adapter_splitter_status_output.out/rx_st_status_output_delay.in
    } else {
        add_connection rx_st_error_adapter_stat.out/rx_st_status_output_delay.in
    }
    
    # Clock Connections
    add_connection ${rx_clock_src}/rx_eth_pkt_backpressure_control.clock_reset
    add_connection ${rx_reset_src}/rx_eth_pkt_backpressure_control.clock_reset_reset
    add_connection ${rx_clock_src}/rx_eth_crc_checker.clock_reset
    add_connection ${rx_reset_src}/rx_eth_crc_checker.clock_reset_reset
    add_connection ${rx_clock_src}/rx_eth_frame_decoder.clock_reset
    add_connection ${rx_reset_src}/rx_eth_frame_decoder.clock_reset_reset
    add_connection ${rx_clock_src}/rx_st_frame_status_splitter.clk
    #add_connection ${rx_reset_src}/rx_st_frame_status_splitter.reset
    add_connection ${rx_clock_src}/rx_st_timing_adapter_frame_status_in.clk
    add_connection ${rx_reset_src}/rx_st_timing_adapter_frame_status_in.reset
    add_connection ${rx_clock_src}/rx_timing_adapter_frame_status_out_frame_decoder.clk
    add_connection ${rx_reset_src}/rx_timing_adapter_frame_status_out_frame_decoder.reset
    add_connection ${rx_clock_src}/rx_timing_adapter_frame_status_out_crc_checker.clk
    add_connection ${rx_reset_src}/rx_timing_adapter_frame_status_out_crc_checker.reset
    add_connection ${rx_clock_src}/rx_eth_frame_status_merger.clock_reset
    add_connection ${rx_reset_src}/rx_eth_frame_status_merger.clock_reset_reset
    if {$preamble_mode == 1} {
		add_connection ${rx_clock_src}/rx_preamble_inserter.clock
		add_connection ${rx_reset_src}/rx_preamble_inserter.reset
		add_connection ${rx_clock_src}/rx_st_status_splitter_2.clk
		add_connection ${rx_clock_src}/rx_st_status_output_delay_final.clk
		add_connection ${rx_reset_src}/rx_st_status_output_delay_final.clk_reset
	}
	
	add_connection ${rx_clock_src}/rx_eth_crc_pad_rem.clock_reset
    add_connection ${rx_reset_src}/rx_eth_crc_pad_rem.clock_reset_reset
    add_connection ${rx_clock_src}/rx_eth_packet_overflow_control.clock_reset
    add_connection ${rx_reset_src}/rx_eth_packet_overflow_control.clock_reset_reset
    add_connection ${rx_clock_src}/rx_st_error_adapter_stat.clk
    add_connection ${rx_reset_src}/rx_st_error_adapter_stat.reset
    if {$stat_ena == 1} {
        add_connection ${rx_clock_src}/rx_st_timing_adapter_splitter_status_in.clk
        add_connection ${rx_reset_src}/rx_st_timing_adapter_splitter_status_in.reset
        add_connection ${rx_clock_src}/rx_st_status_splitter.clk
        #add_connection ${rx_reset_src}/rx_st_status_splitter.reset
        add_connection ${rx_clock_src}/rx_st_timing_adapter_splitter_status_output.clk
        add_connection ${rx_reset_src}/rx_st_timing_adapter_splitter_status_output.reset
        add_connection ${rx_clock_src}/rx_st_timing_adapter_splitter_status_statistics.clk
        add_connection ${rx_reset_src}/rx_st_timing_adapter_splitter_status_statistics.reset
        add_connection ${rx_clock_src}/rx_st_status_statistics_delay.clk
        add_connection ${rx_reset_src}/rx_st_status_statistics_delay.clk_reset
        add_connection ${rx_clock_src}/rx_eth_statistics_collector.clock_reset
        add_connection ${rx_reset_src}/rx_eth_statistics_collector.clock_reset_reset
    }
    add_connection ${rx_clock_src}/rx_st_status_output_delay.clk
    add_connection ${rx_reset_src}/rx_st_status_output_delay.clk_reset
    if {$pfc_ena == 1} {
        add_connection ${rx_clock_src}/rx_eth_pfc_pause_conversion.clock_reset
        add_connection ${rx_reset_src}/rx_eth_pfc_pause_conversion.clock_reset_reset
        
        add_connection ${rx_clock_src}/rx_st_delay_pfc_crc.clk
        add_connection ${rx_reset_src}/rx_st_delay_pfc_crc.clk_reset
        
        add_connection ${rx_clock_src}/rx_eth_delay_pfc_status.clk
        add_connection ${rx_reset_src}/rx_eth_delay_pfc_status.clk_reset
    }
    
    # CSR Connections
    add_connection ${rx_csr_master}/rx_eth_pkt_backpressure_control.csr
    set_connection_parameter_value ${rx_csr_master}/rx_eth_pkt_backpressure_control.csr arbitrationPriority "1"
    set_connection_parameter_value ${rx_csr_master}/rx_eth_pkt_backpressure_control.csr baseAddress "0x0000"
    
    add_connection ${rx_csr_master}/rx_eth_crc_pad_rem.csr
    set_connection_parameter_value ${rx_csr_master}/rx_eth_crc_pad_rem.csr arbitrationPriority "1"
    set_connection_parameter_value ${rx_csr_master}/rx_eth_crc_pad_rem.csr baseAddress "0x0100"
    
    add_connection ${rx_csr_master}/rx_eth_crc_checker.csr
    set_connection_parameter_value ${rx_csr_master}/rx_eth_crc_checker.csr arbitrationPriority "1"
    set_connection_parameter_value ${rx_csr_master}/rx_eth_crc_checker.csr baseAddress "0x0200"
    
    add_connection ${rx_csr_master}/rx_eth_frame_decoder.avalom_mm_csr
    set_connection_parameter_value ${rx_csr_master}/rx_eth_frame_decoder.avalom_mm_csr arbitrationPriority "1"
    set_connection_parameter_value ${rx_csr_master}/rx_eth_frame_decoder.avalom_mm_csr baseAddress "0x02000"
    
    add_connection ${rx_csr_master}/rx_eth_packet_overflow_control.csr
    set_connection_parameter_value ${rx_csr_master}/rx_eth_packet_overflow_control.csr arbitrationPriority "1"
    set_connection_parameter_value ${rx_csr_master}/rx_eth_packet_overflow_control.csr baseAddress "0x0300"
    
    if {$stat_ena == 1} {
        add_connection ${rx_csr_master}/rx_eth_statistics_collector.csr
        set_connection_parameter_value ${rx_csr_master}/rx_eth_statistics_collector.csr arbitrationPriority "1"
        set_connection_parameter_value ${rx_csr_master}/rx_eth_statistics_collector.csr baseAddress "0x3000"
    }
	
	add_connection ${rx_csr_master}/rx_eth_lane_decoder.csr
	set_connection_parameter_value ${rx_csr_master}/rx_eth_lane_decoder.csr arbitrationPriority "1"
	set_connection_parameter_value ${rx_csr_master}/rx_eth_lane_decoder.csr baseAddress "0x0500"
	
	if {$preamble_mode == 1} {
		add_connection ${rx_csr_master}/rx_preamble_inserter.csr
        set_connection_parameter_value ${rx_csr_master}/rx_preamble_inserter.csr arbitrationPriority "1"
        set_connection_parameter_value ${rx_csr_master}/rx_preamble_inserter.csr baseAddress "0x400"
	}
	
}

proc compose_eth_mac_tx {tx_clock_src tx_reset_src tx_csr_master stat_ena reg_stat crc_tx preamble_mode pfc_ena pfc_priority_num} {
    #  Underflow Controller TX
    add_instance tx_eth_packet_underflow_control altera_eth_packet_underflow_control
    set_instance_parameter tx_eth_packet_underflow_control BITSPERSYMBOL "8"
    set_instance_parameter tx_eth_packet_underflow_control SYMBOLSPERBEAT "8"
    set_instance_parameter tx_eth_packet_underflow_control ERROR_WIDTH "1"

    #  Pad Inserter TX
    add_instance tx_eth_pad_inserter altera_eth_pad_inserter
    set_instance_parameter tx_eth_pad_inserter BITSPERSYMBOL "8"
    set_instance_parameter tx_eth_pad_inserter SYMBOLSPERBEAT "8"
    set_instance_parameter tx_eth_pad_inserter ERROR_WIDTH "2"
    
    # Connections
    add_connection tx_eth_packet_underflow_control.avalon_streaming_source/tx_eth_pad_inserter.avalon_st_sink_data
    
    #  Back Pressure Controller TX
    add_instance tx_eth_pkt_backpressure_control altera_eth_pkt_backpressure_control
    set_instance_parameter tx_eth_pkt_backpressure_control BITSPERSYMBOL "8"
    set_instance_parameter tx_eth_pkt_backpressure_control SYMBOLSPERBEAT "8"
    set_instance_parameter tx_eth_pkt_backpressure_control ERROR_WIDTH "2"
    set_instance_parameter tx_eth_pkt_backpressure_control USE_READY "1"
    set_instance_parameter tx_eth_pkt_backpressure_control USE_PAUSE "1"
    
    #  Pause Beat Conversion TX
    add_instance tx_eth_pause_beat_conversion altera_eth_pause_beat_conversion
    add_connection tx_eth_pause_beat_conversion.pause_beat_src/tx_eth_pkt_backpressure_control.avalon_st_pause
    
    # Connections
    if {[_get_device_type]} {
        #  Pipeline Stage TX ST
        add_instance tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control altera_avalon_st_pipeline_stage
        set_instance_parameter tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control SYMBOLS_PER_BEAT "8"
        set_instance_parameter tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control BITS_PER_SYMBOL "8"
        set_instance_parameter tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control USE_PACKETS "1"
        set_instance_parameter tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control USE_EMPTY "1"
        set_instance_parameter tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control CHANNEL_WIDTH "0"
        set_instance_parameter tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control ERROR_WIDTH "2"
        set_instance_parameter tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control PIPELINE_READY "1"
        
        # Connections
        add_connection tx_eth_pad_inserter.avalon_st_source_data/tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control.sink0
        add_connection tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control.source0/tx_eth_pkt_backpressure_control.avalon_st_sink_data
    } else {
        add_connection tx_eth_pad_inserter.avalon_st_source_data/tx_eth_pkt_backpressure_control.avalon_st_sink_data
    }
    
    # Pause Controller TX
    add_instance tx_eth_pause_ctrl_gen altera_eth_pause_ctrl_gen
    set_instance_parameter tx_eth_pause_ctrl_gen BITSPERSYMBOL "8"
    set_instance_parameter tx_eth_pause_ctrl_gen SYMBOLSPERBEAT "8"
    set_instance_parameter tx_eth_pause_ctrl_gen ERROR_WIDTH "1"
    
    #  Error Adapter for Pause Controller TX
    add_instance tx_st_pause_ctrl_error_adapter error_adapter
    set_instance_parameter tx_st_pause_ctrl_error_adapter generationLanguage "VERILOG"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inBitsPerSymbol "8"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inChannelWidth "0"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inErrorDescriptor ""
    set_instance_parameter tx_st_pause_ctrl_error_adapter inErrorWidth "1"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inMaxChannel "0"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inReadyLatency "0"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inSymbolsPerBeat "8"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inUseEmpty "true"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inUseEmptyPort "AUTO"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inUsePackets "true"
    set_instance_parameter tx_st_pause_ctrl_error_adapter inUseReady "true"
    set_instance_parameter tx_st_pause_ctrl_error_adapter moduleName ""
    set_instance_parameter tx_st_pause_ctrl_error_adapter outErrorDescriptor ""
    set_instance_parameter tx_st_pause_ctrl_error_adapter outErrorWidth "2"
    
    # Avalon ST Multiplexer for TX Flow Control and User Frames
    add_instance tx_st_mux_flow_control_user_frame multiplexer
    set_instance_parameter tx_st_mux_flow_control_user_frame bitsPerSymbol "8"
    set_instance_parameter tx_st_mux_flow_control_user_frame errorWidth "2"
    set_instance_parameter tx_st_mux_flow_control_user_frame generationLanguage "VERILOG"
    set_instance_parameter tx_st_mux_flow_control_user_frame moduleName ""
    set_instance_parameter tx_st_mux_flow_control_user_frame numInputInterfaces "2"
    set_instance_parameter tx_st_mux_flow_control_user_frame outChannelWidth "1"
    set_instance_parameter tx_st_mux_flow_control_user_frame packetScheduling "true"
    set_instance_parameter tx_st_mux_flow_control_user_frame schedulingSize "2"
    set_instance_parameter tx_st_mux_flow_control_user_frame symbolsPerBeat "8"
    set_instance_parameter tx_st_mux_flow_control_user_frame useHighBitsOfChannel "true"
    set_instance_parameter tx_st_mux_flow_control_user_frame usePackets "true"
    
    # Channel Adapter TX
    add_instance tx_st_channel_adapter_mux_crc altera_eth_channel_adapter
    set_instance_parameter tx_st_channel_adapter_mux_crc ERROR_WIDTH 3
    
    if { $pfc_ena == 1 } {
        # PFC Generator TX
        add_instance tx_eth_pfc_gen altera_eth_pfc_generator
        set_instance_parameter tx_eth_pfc_gen PFC_PRIORITY_NUM $pfc_priority_num
        
        # Avalon ST Multiplexer for TX Flow Control and TX PFC
        add_instance tx_st_mux_fc_pfc multiplexer
        set_instance_parameter tx_st_mux_fc_pfc bitsPerSymbol "8"
        set_instance_parameter tx_st_mux_fc_pfc errorWidth "1"
        set_instance_parameter tx_st_mux_fc_pfc generationLanguage "VERILOG"
        set_instance_parameter tx_st_mux_fc_pfc moduleName ""
        set_instance_parameter tx_st_mux_fc_pfc numInputInterfaces "2"
        set_instance_parameter tx_st_mux_fc_pfc outChannelWidth "1"
        set_instance_parameter tx_st_mux_fc_pfc packetScheduling "true"
        set_instance_parameter tx_st_mux_fc_pfc schedulingSize "2"
        set_instance_parameter tx_st_mux_fc_pfc symbolsPerBeat "8"
        set_instance_parameter tx_st_mux_fc_pfc useHighBitsOfChannel "true"
        set_instance_parameter tx_st_mux_fc_pfc usePackets "true"
        
        # Channel Adapter TX FC PFC
        add_instance tx_st_channel_adapter_mux_fc_pfc altera_eth_channel_adapter
        set_instance_parameter tx_st_channel_adapter_mux_fc_pfc ERROR_WIDTH 1
        
        # Connections
        add_connection tx_eth_pause_ctrl_gen.pause_packet/tx_st_mux_fc_pfc.in0
        add_connection tx_eth_pfc_gen.pfc_data_src/tx_st_mux_fc_pfc.in1
        
        add_connection tx_st_mux_fc_pfc.out/tx_st_channel_adapter_mux_fc_pfc.channel_adapter_sink
        add_connection tx_st_channel_adapter_mux_fc_pfc.channel_adapter_src/tx_st_pause_ctrl_error_adapter.in
    } else {
        add_connection tx_eth_pause_ctrl_gen.pause_packet/tx_st_pause_ctrl_error_adapter.in
    }
    
    # Connections
	add_connection tx_eth_pkt_backpressure_control.avalon_st_source_data/tx_st_mux_flow_control_user_frame.in0
    add_connection tx_st_pause_ctrl_error_adapter.out/tx_st_mux_flow_control_user_frame.in1
	
    if {$preamble_mode == 0 } {
        add_connection tx_st_mux_flow_control_user_frame.out/tx_eth_address_inserter.avalon_streaming_sink
	} else {
        #  Avalon ST Splitter TX 2
        add_instance tx_st_splitter_2 altera_avalon_st_splitter 
        set_instance_parameter tx_st_splitter_2 NUMBER_OF_OUTPUTS "2"
        set_instance_parameter tx_st_splitter_2 QUALIFY_VALID_OUT "1"
        set_instance_parameter tx_st_splitter_2 DATA_WIDTH "64"
        set_instance_parameter tx_st_splitter_2 BITS_PER_SYMBOL "8"
        set_instance_parameter tx_st_splitter_2 USE_PACKETS "1"
        set_instance_parameter tx_st_splitter_2 USE_CHANNEL "1"
        set_instance_parameter tx_st_splitter_2 CHANNEL_WIDTH "1"
        set_instance_parameter tx_st_splitter_2 MAX_CHANNELS "1"
        set_instance_parameter tx_st_splitter_2 USE_ERROR "1"
        set_instance_parameter tx_st_splitter_2 ERROR_WIDTH "2"
        
        # Connections
        add_connection tx_st_mux_flow_control_user_frame.out/tx_st_splitter_2.in
        add_connection tx_st_splitter_2.out1/tx_preamble_passthrough.avalon_streaming_sink_channel_info
        add_connection tx_st_splitter_2.out0/tx_eth_address_inserter.avalon_streaming_sink
	}
	
    
    
    #  Address Inserter TX
    add_instance tx_eth_address_inserter altera_eth_address_inserter
    set_instance_parameter tx_eth_address_inserter BITSPERSYMBOL "8"
    set_instance_parameter tx_eth_address_inserter SYMBOLSPERBEAT "8"
    set_instance_parameter tx_eth_address_inserter ERROR_WIDTH "2"
    
    if {$crc_tx == 1} {
        #  CRC Inserter TX
        add_instance tx_eth_crc_inserter altera_eth_crc
        set_instance_parameter tx_eth_crc_inserter BITSPERSYMBOL "8"
        set_instance_parameter tx_eth_crc_inserter SYMBOLSPERBEAT "8"
        set_instance_parameter tx_eth_crc_inserter ERROR_WIDTH "2"
        set_instance_parameter tx_eth_crc_inserter MODE_CHECKER_0_INSERTER_1 "1"
        set_instance_parameter tx_eth_crc_inserter USE_READY "true"
        set_instance_parameter tx_eth_crc_inserter USE_CHANNEL "1"
        # Connections
        add_connection tx_eth_address_inserter.avalon_streaming_source/tx_eth_crc_inserter.avalon_streaming_sink
        add_connection tx_eth_crc_inserter.avalon_streaming_source/tx_st_channel_adapter_mux_crc.channel_adapter_sink
        add_connection tx_st_channel_adapter_mux_crc.channel_adapter_src/tx_st_pipeline_stage_rs.sink0
        
    } else {
        # Error Adapter NO_TX_CRC
        add_instance tx_st_error_adapter_no_crc error_adapter
        set_instance_parameter tx_st_error_adapter_no_crc generationLanguage "VERILOG"
        set_instance_parameter tx_st_error_adapter_no_crc inBitsPerSymbol "8"
        set_instance_parameter tx_st_error_adapter_no_crc inChannelWidth "1"
        set_instance_parameter tx_st_error_adapter_no_crc inErrorDescriptor ""
        set_instance_parameter tx_st_error_adapter_no_crc inErrorWidth "2"
        set_instance_parameter tx_st_error_adapter_no_crc inMaxChannel "1"
        set_instance_parameter tx_st_error_adapter_no_crc inReadyLatency "0"
        set_instance_parameter tx_st_error_adapter_no_crc inSymbolsPerBeat "8"
        set_instance_parameter tx_st_error_adapter_no_crc inUseEmpty "true"
        set_instance_parameter tx_st_error_adapter_no_crc inUseEmptyPort "AUTO"
        set_instance_parameter tx_st_error_adapter_no_crc inUsePackets "true"
        set_instance_parameter tx_st_error_adapter_no_crc inUseReady "true"
        set_instance_parameter tx_st_error_adapter_no_crc moduleName ""
        set_instance_parameter tx_st_error_adapter_no_crc outErrorDescriptor ""
        set_instance_parameter tx_st_error_adapter_no_crc outErrorWidth "3"
        
        # Connections
        add_connection tx_eth_address_inserter.avalon_streaming_source/tx_st_error_adapter_no_crc.in
        add_connection tx_st_error_adapter_no_crc.out/tx_st_channel_adapter_mux_crc.channel_adapter_sink
        add_connection tx_st_channel_adapter_mux_crc.channel_adapter_src/tx_st_pipeline_stage_rs.sink0        
        
    }
    
    #  Pipeline Stage TX RS
    add_instance tx_st_pipeline_stage_rs altera_avalon_st_pipeline_stage
    set_instance_parameter tx_st_pipeline_stage_rs SYMBOLS_PER_BEAT "8"
    set_instance_parameter tx_st_pipeline_stage_rs BITS_PER_SYMBOL "8"
    set_instance_parameter tx_st_pipeline_stage_rs USE_PACKETS "1"
    set_instance_parameter tx_st_pipeline_stage_rs USE_EMPTY "1"
    set_instance_parameter tx_st_pipeline_stage_rs CHANNEL_WIDTH "0"
    set_instance_parameter tx_st_pipeline_stage_rs ERROR_WIDTH "3"
    set_instance_parameter tx_st_pipeline_stage_rs PIPELINE_READY "1"
    
    # Statistics on TX
    if {$stat_ena == 1} {
		if {$preamble_mode == 1} {
            #  Avalon ST Splitter TX 1
            add_instance tx_st_splitter_1 altera_avalon_st_splitter 
            set_instance_parameter tx_st_splitter_1 NUMBER_OF_OUTPUTS "3"
            set_instance_parameter tx_st_splitter_1 QUALIFY_VALID_OUT "1"
            set_instance_parameter tx_st_splitter_1 DATA_WIDTH "64"
            set_instance_parameter tx_st_splitter_1 BITS_PER_SYMBOL "8"
            set_instance_parameter tx_st_splitter_1 USE_PACKETS "1"
            set_instance_parameter tx_st_splitter_1 USE_CHANNEL "0"
            set_instance_parameter tx_st_splitter_1 CHANNEL_WIDTH "1"
            set_instance_parameter tx_st_splitter_1 MAX_CHANNELS "1"
            set_instance_parameter tx_st_splitter_1 USE_ERROR "1"
            set_instance_parameter tx_st_splitter_1 ERROR_WIDTH "3"
		} else {
            #  Avalon ST Splitter TX 1
            add_instance tx_st_splitter_1 altera_avalon_st_splitter 
            set_instance_parameter tx_st_splitter_1 NUMBER_OF_OUTPUTS "2"
            set_instance_parameter tx_st_splitter_1 QUALIFY_VALID_OUT "1"
            set_instance_parameter tx_st_splitter_1 DATA_WIDTH "64"
            set_instance_parameter tx_st_splitter_1 BITS_PER_SYMBOL "8"
            set_instance_parameter tx_st_splitter_1 USE_PACKETS "1"
            set_instance_parameter tx_st_splitter_1 USE_CHANNEL "0"
            set_instance_parameter tx_st_splitter_1 CHANNEL_WIDTH "1"
            set_instance_parameter tx_st_splitter_1 MAX_CHANNELS "1"
            set_instance_parameter tx_st_splitter_1 USE_ERROR "1"
            set_instance_parameter tx_st_splitter_1 ERROR_WIDTH "3"
		}
        
        #  Timing Adapter for Frame Decoder
        add_instance tx_st_timing_adapter_frame_decoder timing_adapter
        set_instance_parameter tx_st_timing_adapter_frame_decoder generationLanguage "VERILOG"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inBitsPerSymbol "8"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inChannelWidth "0"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inErrorDescriptor ""
        set_instance_parameter tx_st_timing_adapter_frame_decoder inErrorWidth "3"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inMaxChannel "0"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inSymbolsPerBeat "8"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inUseEmpty "true"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inUseEmptyPort "YES"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inUsePackets "true"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inUseReady "true"
        set_instance_parameter tx_st_timing_adapter_frame_decoder inUseValid "true"
        set_instance_parameter tx_st_timing_adapter_frame_decoder moduleName ""
        set_instance_parameter tx_st_timing_adapter_frame_decoder outReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_frame_decoder outUseReady "false"
        set_instance_parameter tx_st_timing_adapter_frame_decoder outUseValid "true"
        
        #  Frame Decoder TX
        add_instance tx_eth_frame_decoder altera_eth_frame_decoder
        set_instance_parameter tx_eth_frame_decoder BITSPERSYMBOL "8"
        set_instance_parameter tx_eth_frame_decoder SYMBOLSPERBEAT "8"
        set_instance_parameter tx_eth_frame_decoder ERROR_WIDTH "3"
        set_instance_parameter tx_eth_frame_decoder ENABLE_SUPP_ADDR "0"
        set_instance_parameter tx_eth_frame_decoder ENABLE_DATA_SOURCE "0"
        set_instance_parameter tx_eth_frame_decoder ENABLE_PAUSELEN "0"
        set_instance_parameter tx_eth_frame_decoder ENABLE_PKTINFO "0"
        set_instance_parameter tx_eth_frame_decoder USE_READY "0"
        set_instance_parameter tx_eth_frame_decoder ENABLE_PFC $pfc_ena
        set_instance_parameter tx_eth_frame_decoder PFC_PRIORITY_NUM $pfc_priority_num
        set_instance_parameter tx_eth_frame_decoder ENABLE_PFC_PQ "0"
        set_instance_parameter tx_eth_frame_decoder ENABLE_PFC_STATUS "0"

        #  Error Adapter TX for Statistics
        add_instance tx_st_error_adapter_stat error_adapter
        set_instance_parameter tx_st_error_adapter_stat generationLanguage "VERILOG"
        set_instance_parameter tx_st_error_adapter_stat inBitsPerSymbol "40"
        set_instance_parameter tx_st_error_adapter_stat inChannelWidth "0"
        set_instance_parameter tx_st_error_adapter_stat inErrorDescriptor "payload_length,oversize,undersize,crc,underflow,user"
        set_instance_parameter tx_st_error_adapter_stat inErrorWidth "6"
        set_instance_parameter tx_st_error_adapter_stat inMaxChannel "0"
        set_instance_parameter tx_st_error_adapter_stat inReadyLatency "0"
        set_instance_parameter tx_st_error_adapter_stat inSymbolsPerBeat "1"
        set_instance_parameter tx_st_error_adapter_stat inUseEmpty "false"
        set_instance_parameter tx_st_error_adapter_stat inUseEmptyPort "AUTO"
        set_instance_parameter tx_st_error_adapter_stat inUsePackets "false"
        set_instance_parameter tx_st_error_adapter_stat inUseReady "false"
        set_instance_parameter tx_st_error_adapter_stat moduleName ""
        set_instance_parameter tx_st_error_adapter_stat outErrorDescriptor "phy,user,underflow,crc,payload_length,oversize,undersize"
        set_instance_parameter tx_st_error_adapter_stat outErrorWidth "7"

		#  Timing Adapter for Status Splitter Input
        add_instance tx_st_timing_adapter_splitter_status_in timing_adapter
        set_instance_parameter tx_st_timing_adapter_splitter_status_in generationLanguage "VERILOG"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inBitsPerSymbol "40"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inChannelWidth "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inErrorDescriptor ""
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inErrorWidth "7"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inMaxChannel "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inSymbolsPerBeat "1"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inUseEmpty "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inUseEmptyPort "YES"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inUsePackets "false"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inUseReady "false"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in inUseValid "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in moduleName ""
        set_instance_parameter tx_st_timing_adapter_splitter_status_in outReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in outUseReady "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_in outUseValid "true"
		
		#  Timing Adapter for Statistics of Status Splitter
        add_instance tx_st_timing_adapter_splitter_status_output timing_adapter
        set_instance_parameter tx_st_timing_adapter_splitter_status_output generationLanguage "VERILOG"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inBitsPerSymbol "40"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inChannelWidth "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inErrorDescriptor ""
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inErrorWidth "7"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inMaxChannel "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inSymbolsPerBeat "1"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inUseEmpty "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inUseEmptyPort "YES"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inUsePackets "false"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inUseReady "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output inUseValid "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output moduleName ""
        set_instance_parameter tx_st_timing_adapter_splitter_status_output outReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output outUseReady "false"
        set_instance_parameter tx_st_timing_adapter_splitter_status_output outUseValid "true"
		
		# Splitter for TX Status
        add_instance tx_st_status_splitter altera_avalon_st_splitter 
        set_instance_parameter tx_st_status_splitter NUMBER_OF_OUTPUTS "2"
        set_instance_parameter tx_st_status_splitter QUALIFY_VALID_OUT "1"
        set_instance_parameter tx_st_status_splitter DATA_WIDTH "40"
        set_instance_parameter tx_st_status_splitter BITS_PER_SYMBOL "40"
        set_instance_parameter tx_st_status_splitter USE_PACKETS "0"
        set_instance_parameter tx_st_status_splitter USE_CHANNEL "0"
        set_instance_parameter tx_st_status_splitter CHANNEL_WIDTH "1"
        set_instance_parameter tx_st_status_splitter MAX_CHANNELS "1"
        set_instance_parameter tx_st_status_splitter USE_ERROR "1"
        set_instance_parameter tx_st_status_splitter ERROR_WIDTH "7"	
	   
		#  Timing Adapter for Statistics of Status Splitter
        add_instance tx_st_timing_adapter_splitter_status_statistics timing_adapter
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics generationLanguage "VERILOG"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inBitsPerSymbol "40"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inChannelWidth "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inErrorDescriptor ""
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inErrorWidth "7"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inMaxChannel "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inSymbolsPerBeat "1"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inUseEmpty "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inUseEmptyPort "YES"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inUsePackets "false"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inUseReady "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics inUseValid "true"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics moduleName ""
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics outReadyLatency "0"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics outUseReady "false"
        set_instance_parameter tx_st_timing_adapter_splitter_status_statistics outUseValid "true"
		
        if {$reg_stat == 1} {
            # Statistics Collector TX Register Based
            add_instance tx_eth_statistics_collector altera_eth_statistics_collector
            set_instance_parameter tx_eth_statistics_collector ENABLE_PFC $pfc_ena
        } else {
            # Statistics Collector TX Memory Based
            add_instance tx_eth_statistics_collector altera_eth_10gmem_statistics_collector
            set_instance_parameter tx_eth_statistics_collector ENABLE_PFC $pfc_ena
        }
    } else {
		if {$preamble_mode == 1} {
			#  Avalon ST Splitter TX 1
			add_instance tx_st_splitter_1 altera_avalon_st_splitter 
			set_instance_parameter tx_st_splitter_1 NUMBER_OF_OUTPUTS "2"
			set_instance_parameter tx_st_splitter_1 QUALIFY_VALID_OUT "1"
			set_instance_parameter tx_st_splitter_1 DATA_WIDTH "64"
			set_instance_parameter tx_st_splitter_1 BITS_PER_SYMBOL "8"
			set_instance_parameter tx_st_splitter_1 USE_PACKETS "1"
			set_instance_parameter tx_st_splitter_1 USE_CHANNEL "0"
			set_instance_parameter tx_st_splitter_1 CHANNEL_WIDTH "1"
			set_instance_parameter tx_st_splitter_1 MAX_CHANNELS "1"
			set_instance_parameter tx_st_splitter_1 USE_ERROR "1"
			set_instance_parameter tx_st_splitter_1 ERROR_WIDTH "3"
		}	
	}
    
    if {$stat_ena == 1} {
		add_connection tx_st_pipeline_stage_rs.source0/tx_st_splitter_1.in
		if {$preamble_mode == 1} {
			add_connection tx_st_splitter_1.out0/tx_st_timing_adapter_frame_decoder.in
			add_connection tx_st_splitter_1.out1/tx_eth_packet_formatter.data_sink
			add_connection tx_st_splitter_1.out2/tx_preamble_passthrough.avalon_streaming_sink_pkt_f
		} else {
			add_connection tx_st_splitter_1.out0/tx_st_timing_adapter_frame_decoder.in
			add_connection tx_st_splitter_1.out1/tx_eth_packet_formatter.data_sink
		}
        add_connection tx_st_timing_adapter_frame_decoder.out/tx_eth_frame_decoder.avalon_st_data_sink
        add_connection tx_eth_frame_decoder.avalon_st_rxstatus_src/tx_st_error_adapter_stat.in

		add_connection tx_st_error_adapter_stat.out/tx_st_timing_adapter_splitter_status_in.in
        add_connection tx_st_timing_adapter_splitter_status_in.out/tx_st_status_splitter.in
        add_connection tx_st_status_splitter.out0/tx_st_timing_adapter_splitter_status_statistics.in
        add_connection tx_st_timing_adapter_splitter_status_statistics.out/tx_eth_statistics_collector.avalon_st_sink_data
		
        
		add_connection tx_st_status_splitter.out1/tx_st_timing_adapter_splitter_status_output.in

		
    } else {
		if {$preamble_mode == 1} {
		add_connection tx_st_pipeline_stage_rs.source0/tx_st_splitter_1.in
		add_connection tx_st_splitter_1.out0/tx_eth_packet_formatter.data_sink
        add_connection tx_st_splitter_1.out1/tx_preamble_passthrough.avalon_streaming_sink_pkt_f	
		} else {
		add_connection tx_st_pipeline_stage_rs.source0/tx_eth_packet_formatter.data_sink
		}		
	}
    
    # Clock Connections
    add_connection ${tx_clock_src}/tx_eth_packet_underflow_control.clock_reset
    add_connection ${tx_reset_src}/tx_eth_packet_underflow_control.clock_reset_reset
    add_connection ${tx_clock_src}/tx_eth_pad_inserter.clock_reset
    add_connection ${tx_reset_src}/tx_eth_pad_inserter.clock_reset_reset
    add_connection ${tx_clock_src}/tx_eth_pkt_backpressure_control.clock_reset
    add_connection ${tx_reset_src}/tx_eth_pkt_backpressure_control.clock_reset_reset
    add_connection ${tx_clock_src}/tx_eth_pause_beat_conversion.clock_reset
    add_connection ${tx_reset_src}/tx_eth_pause_beat_conversion.clock_reset_reset
    
    if {[_get_device_type]} {
        add_connection ${tx_clock_src}/tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control.cr0
        add_connection ${tx_reset_src}/tx_st_pipeline_stage_pad_inserter_pkt_backpressure_control.cr0_reset
    }
    
    add_connection ${tx_clock_src}/tx_eth_pause_ctrl_gen.clock_reset
    add_connection ${tx_reset_src}/tx_eth_pause_ctrl_gen.clock_reset_reset
    add_connection ${tx_clock_src}/tx_st_pause_ctrl_error_adapter.clk
    add_connection ${tx_reset_src}/tx_st_pause_ctrl_error_adapter.reset
    add_connection ${tx_clock_src}/tx_st_mux_flow_control_user_frame.clk
    add_connection ${tx_reset_src}/tx_st_mux_flow_control_user_frame.reset
    add_connection ${tx_clock_src}/tx_st_channel_adapter_mux_crc.clock_reset
    add_connection ${tx_reset_src}/tx_st_channel_adapter_mux_crc.clock_reset_reset
    add_connection ${tx_clock_src}/tx_eth_address_inserter.clock_reset
    add_connection ${tx_reset_src}/tx_eth_address_inserter.clock_reset_reset
    
    if {$crc_tx == 1} {
        add_connection ${tx_clock_src}/tx_eth_crc_inserter.clock_reset
        add_connection ${tx_reset_src}/tx_eth_crc_inserter.clock_reset_reset
    } else {
        add_connection ${tx_clock_src}/tx_st_error_adapter_no_crc.clk
        add_connection ${tx_reset_src}/tx_st_error_adapter_no_crc.reset
    }
    add_connection ${tx_clock_src}/tx_st_pipeline_stage_rs.cr0
    add_connection ${tx_reset_src}/tx_st_pipeline_stage_rs.cr0_reset
    
	
    if {$stat_ena == 1} {
		add_connection ${tx_clock_src}/tx_st_splitter_1.clk
		#add_connection ${tx_reset_src}/tx_st_splitter_1.reset	
        add_connection ${tx_clock_src}/tx_st_timing_adapter_frame_decoder.clk
        add_connection ${tx_reset_src}/tx_st_timing_adapter_frame_decoder.reset
        add_connection ${tx_clock_src}/tx_eth_frame_decoder.clock_reset
        add_connection ${tx_reset_src}/tx_eth_frame_decoder.clock_reset_reset
        add_connection ${tx_clock_src}/tx_st_error_adapter_stat.clk
        add_connection ${tx_reset_src}/tx_st_error_adapter_stat.reset
        add_connection ${tx_clock_src}/tx_st_timing_adapter_splitter_status_in.clk
        add_connection ${tx_reset_src}/tx_st_timing_adapter_splitter_status_in.reset
		add_connection ${tx_clock_src}/tx_st_timing_adapter_splitter_status_output.clk
        add_connection ${tx_reset_src}/tx_st_timing_adapter_splitter_status_output.reset
		add_connection ${tx_clock_src}/tx_st_status_splitter.clk
		add_connection ${tx_clock_src}/tx_st_timing_adapter_splitter_status_statistics.clk
        add_connection ${tx_reset_src}/tx_st_timing_adapter_splitter_status_statistics.reset
		
        add_connection ${tx_clock_src}/tx_eth_statistics_collector.clock_reset
        add_connection ${tx_reset_src}/tx_eth_statistics_collector.clock_reset_reset
    } else {
		if {$preamble_mode == 1} {
			add_connection ${tx_clock_src}/tx_st_splitter_1.clk
		}
	}
    
    if {$pfc_ena == 1} {
        add_connection ${tx_clock_src}/tx_eth_pfc_gen.clock_reset
        add_connection ${tx_reset_src}/tx_eth_pfc_gen.clock_reset_reset
        add_connection ${tx_clock_src}/tx_st_mux_fc_pfc.clk
        add_connection ${tx_reset_src}/tx_st_mux_fc_pfc.reset
        add_connection ${tx_clock_src}/tx_st_channel_adapter_mux_fc_pfc.clock_reset
        add_connection ${tx_reset_src}/tx_st_channel_adapter_mux_fc_pfc.clock_reset_reset
    }
    
	if {$preamble_mode == 1} {
		add_connection ${tx_clock_src}/tx_st_splitter_2.clk
	}
	
    # CSR Connections
    add_connection ${tx_csr_master}/tx_eth_pkt_backpressure_control.csr
    set_connection_parameter_value ${tx_csr_master}/tx_eth_pkt_backpressure_control.csr arbitrationPriority "1"
    set_connection_parameter_value ${tx_csr_master}/tx_eth_pkt_backpressure_control.csr baseAddress "0x0"

    add_connection ${tx_csr_master}/tx_eth_pad_inserter.csr
    set_connection_parameter_value ${tx_csr_master}/tx_eth_pad_inserter.csr arbitrationPriority "1"
    set_connection_parameter_value ${tx_csr_master}/tx_eth_pad_inserter.csr baseAddress "0x100"
    
    if {$crc_tx == 1} {
        add_connection ${tx_csr_master}/tx_eth_crc_inserter.csr
        set_connection_parameter_value ${tx_csr_master}/tx_eth_crc_inserter.csr arbitrationPriority "1"
        set_connection_parameter_value ${tx_csr_master}/tx_eth_crc_inserter.csr baseAddress "0x200"
    }
    
    add_connection ${tx_csr_master}/tx_eth_pause_ctrl_gen.csr
    set_connection_parameter_value ${tx_csr_master}/tx_eth_pause_ctrl_gen.csr arbitrationPriority "1"
    set_connection_parameter_value ${tx_csr_master}/tx_eth_pause_ctrl_gen.csr baseAddress "0x500"
    
    add_connection ${tx_csr_master}/tx_eth_address_inserter.csr
    set_connection_parameter_value ${tx_csr_master}/tx_eth_address_inserter.csr arbitrationPriority "1"
    set_connection_parameter_value ${tx_csr_master}/tx_eth_address_inserter.csr baseAddress "0x800"
    
    add_connection ${tx_csr_master}/tx_eth_packet_underflow_control.avalon_slave_0
    set_connection_parameter_value ${tx_csr_master}/tx_eth_packet_underflow_control.avalon_slave_0 arbitrationPriority "1"
    set_connection_parameter_value ${tx_csr_master}/tx_eth_packet_underflow_control.avalon_slave_0 baseAddress "0x0300"
    
    if {$stat_ena == 1} {
        add_connection ${tx_csr_master}/tx_eth_frame_decoder.avalom_mm_csr
        set_connection_parameter_value ${tx_csr_master}/tx_eth_frame_decoder.avalom_mm_csr arbitrationPriority "1"
        set_connection_parameter_value ${tx_csr_master}/tx_eth_frame_decoder.avalom_mm_csr baseAddress "0x02000"
        
        add_connection ${tx_csr_master}/tx_eth_statistics_collector.csr
        set_connection_parameter_value ${tx_csr_master}/tx_eth_statistics_collector.csr arbitrationPriority "1"
        set_connection_parameter_value ${tx_csr_master}/tx_eth_statistics_collector.csr baseAddress "0x03000"
    }
	
    if {$pfc_ena == 1} {
        add_connection ${tx_csr_master}/tx_eth_pfc_gen.csr
        set_connection_parameter_value ${tx_csr_master}/tx_eth_pfc_gen.csr arbitrationPriority "1"
        set_connection_parameter_value ${tx_csr_master}/tx_eth_pfc_gen.csr baseAddress "0x0600"
    }
    
	if {$preamble_mode == 1} {
		add_connection ${tx_csr_master}/tx_preamble_passthrough.csr
		set_connection_parameter_value ${tx_csr_master}/tx_preamble_passthrough.csr arbitrationPriority "1"
        set_connection_parameter_value ${tx_csr_master}/tx_preamble_passthrough.csr baseAddress "0x400"
	}
}

proc compose_eth_xgmii_tx {tx_clock_src tx_reset_src preamble_mode} {
    #  Packet Formatter
    add_instance tx_eth_packet_formatter altera_eth_packet_formatter
    set_instance_parameter tx_eth_packet_formatter ERROR_WIDTH "3"
    set_instance_parameter tx_eth_packet_formatter GET_PREAMBLE $preamble_mode
    #  XGMII Termination
    add_instance tx_eth_xgmii_termination altera_eth_xgmii_termination
    
    #  Link Fault Generation
    add_instance tx_eth_link_fault_generation altera_eth_link_fault_generation
    
    # Clock Connections
    add_connection ${tx_clock_src}/tx_eth_packet_formatter.clk
    add_connection ${tx_reset_src}/tx_eth_packet_formatter.clk_reset
    add_connection ${tx_clock_src}/tx_eth_xgmii_termination.clock_reset
    add_connection ${tx_reset_src}/tx_eth_xgmii_termination.clock_reset_reset
    add_connection ${tx_clock_src}/tx_eth_link_fault_generation.clk
    add_connection ${tx_reset_src}/tx_eth_link_fault_generation.clk_reset
    
    # Connections
	
    add_connection tx_eth_packet_formatter.data_src/tx_eth_xgmii_termination.avalon_streaming_sink
    add_connection tx_eth_xgmii_termination.avalon_streaming_source/tx_eth_link_fault_generation.mii_sink
}

proc compose_eth_xgmii_rx {rx_clock_src rx_reset_src preamble_mode} {
    #  Timing Adapter RX Interface Conversion
    add_instance rx_st_timing_adapter_interface_conversion timing_adapter
    set_instance_parameter rx_st_timing_adapter_interface_conversion generationLanguage "VERILOG"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inBitsPerSymbol "9"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inChannelWidth "0"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inErrorDescriptor ""
    set_instance_parameter rx_st_timing_adapter_interface_conversion inErrorWidth "0"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inMaxChannel "0"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inSymbolsPerBeat "8"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inUseEmpty "false"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inUseEmptyPort "NO"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inUsePackets "false"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inUseReady "false"
    set_instance_parameter rx_st_timing_adapter_interface_conversion inUseValid "false"
    set_instance_parameter rx_st_timing_adapter_interface_conversion moduleName ""
    set_instance_parameter rx_st_timing_adapter_interface_conversion outReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_interface_conversion outUseReady "true"
    set_instance_parameter rx_st_timing_adapter_interface_conversion outUseValid "true"
    
    #  Avalon St Splitter RX XGMII
    add_instance rx_st_splitter_xgmii altera_avalon_st_splitter
    set_instance_parameter rx_st_splitter_xgmii NUMBER_OF_OUTPUTS "2"
    set_instance_parameter rx_st_splitter_xgmii QUALIFY_VALID_OUT "1"
    set_instance_parameter rx_st_splitter_xgmii DATA_WIDTH "72"
    set_instance_parameter rx_st_splitter_xgmii BITS_PER_SYMBOL "9"
    set_instance_parameter rx_st_splitter_xgmii USE_PACKETS "0"
    set_instance_parameter rx_st_splitter_xgmii USE_CHANNEL "0"
    set_instance_parameter rx_st_splitter_xgmii CHANNEL_WIDTH "1"
    set_instance_parameter rx_st_splitter_xgmii MAX_CHANNELS "1"
    set_instance_parameter rx_st_splitter_xgmii USE_ERROR "0"
    set_instance_parameter rx_st_splitter_xgmii ERROR_WIDTH "1"
    
    #  Timing Adapter RX Lane Decoder
    add_instance rx_st_timing_adapter_lane_decoder timing_adapter
    set_instance_parameter rx_st_timing_adapter_lane_decoder generationLanguage "VERILOG"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inBitsPerSymbol "9"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inChannelWidth "0"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inErrorDescriptor ""
    set_instance_parameter rx_st_timing_adapter_lane_decoder inErrorWidth "0"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inMaxChannel "0"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inSymbolsPerBeat "8"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inUseEmpty "false"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inUseEmptyPort "NO"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inUsePackets "false"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inUseReady "true"
    set_instance_parameter rx_st_timing_adapter_lane_decoder inUseValid "true"
    set_instance_parameter rx_st_timing_adapter_lane_decoder moduleName ""
    set_instance_parameter rx_st_timing_adapter_lane_decoder outReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_lane_decoder outUseReady "false"
    set_instance_parameter rx_st_timing_adapter_lane_decoder outUseValid "false"
    
    #  Timing Adapter RX Link Fault Detection
    add_instance rx_st_timing_adapter_link_fault_detection timing_adapter
    set_instance_parameter rx_st_timing_adapter_link_fault_detection generationLanguage "VERILOG"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inBitsPerSymbol "9"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inChannelWidth "0"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inErrorDescriptor ""
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inErrorWidth "0"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inMaxChannel "0"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inSymbolsPerBeat "8"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inUseEmpty "false"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inUseEmptyPort "NO"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inUsePackets "false"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inUseReady "true"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection inUseValid "true"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection moduleName ""
    set_instance_parameter rx_st_timing_adapter_link_fault_detection outReadyLatency "0"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection outUseReady "false"
    set_instance_parameter rx_st_timing_adapter_link_fault_detection outUseValid "false"
    
    #  Link Fault Detection RX
    add_instance rx_eth_link_fault_detection altera_eth_link_fault_detection
    
    #  Lane Decoder RX
    add_instance rx_eth_lane_decoder altera_eth_lane_decoder
	set_instance_parameter rx_eth_lane_decoder PREAMBLE_MODE $preamble_mode
	
    
    # Clock Connections
    add_connection ${rx_clock_src}/rx_st_timing_adapter_interface_conversion.clk
    add_connection ${rx_reset_src}/rx_st_timing_adapter_interface_conversion.reset
    add_connection ${rx_clock_src}/rx_st_splitter_xgmii.clk
    #add_connection ${rx_reset_src}/rx_st_splitter_xgmii.reset
    
    add_connection ${rx_clock_src}/rx_st_timing_adapter_lane_decoder.clk
    add_connection ${rx_reset_src}/rx_st_timing_adapter_lane_decoder.reset
    add_connection ${rx_clock_src}/rx_eth_lane_decoder.clock_reset
    add_connection ${rx_reset_src}/rx_eth_lane_decoder.clock_reset_reset
    
    add_connection ${rx_clock_src}/rx_st_timing_adapter_link_fault_detection.clk
    add_connection ${rx_reset_src}/rx_st_timing_adapter_link_fault_detection.reset
    add_connection ${rx_clock_src}/rx_eth_link_fault_detection.clk
    add_connection ${rx_reset_src}/rx_eth_link_fault_detection.clk_reset
    
    # Connections
    add_connection rx_st_timing_adapter_interface_conversion.out/rx_st_splitter_xgmii.in
    
    add_connection rx_st_splitter_xgmii.out0/rx_st_timing_adapter_lane_decoder.in
    add_connection rx_st_timing_adapter_lane_decoder.out/rx_eth_lane_decoder.avalon_streaming_sink 
    
    add_connection rx_st_splitter_xgmii.out1/rx_st_timing_adapter_link_fault_detection.in
    add_connection rx_st_timing_adapter_link_fault_detection.out/rx_eth_link_fault_detection.mii_sink
}


proc _get_device_type {} {
    set device_family [get_parameter_value DEVICE_FAMILY]
    
    if {![string compare $device_family "Cyclone IV GX"]} {
        return 1;
    } elseif {![string compare $device_family "Arria GX"]} {
        return 1;
    } else {
        return 0;
    }
}
